﻿using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;

namespace HowToGetNotifiedOnCollectionCommitEdit
{
	/// <summary>
	/// This class gives us the control of instantiating a new view model when the user starts typing into the add new record line.
	/// Now we want to do additional actions when the user committed the new record (hit enter). We tried
	/// IEditableCollectionView which provides the methods CommitEdit and CancelEdit which is exactly what we would like
	/// to use, but they are not called when hitting enter or press the escape key.
	/// The question is how can we get notified when a new record has been accepted 
	/// without writing code behind and preferably here in this class?
	/// </summary>
	public class ProductItems : BindingList<ProductItemViewModel>, IList<IProductItemViewModel>, INotifyCollectionChanged
	{
		IProductItemViewModel IList<IProductItemViewModel>.this[int index]
		{
			get => base[index];
			set => base[index] = value as ProductItemViewModel;
		}

		public bool IsReadOnly => false;

		public event NotifyCollectionChangedEventHandler CollectionChanged;

		public void Add(IProductItemViewModel item)
		{
			base.Add(item as ProductItemViewModel);

			CollectionChanged?.Invoke(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Add, Items));
		}

		public bool Contains(IProductItemViewModel item)
		{
			return base.Contains(item as ProductItemViewModel);
		}

		public int IndexOf(IProductItemViewModel item)
		{
			return base.IndexOf(item as ProductItemViewModel);
		}

		public void Insert(int index, IProductItemViewModel item)
		{
			base.Insert(index, item as ProductItemViewModel);

			CollectionChanged?.Invoke(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Remove, Items));
		}

		public bool Remove(IProductItemViewModel item)
		{
			CollectionChanged?.Invoke(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));

			return base.Remove(item as ProductItemViewModel);
		}

		IEnumerator<IProductItemViewModel> IEnumerable<IProductItemViewModel>.GetEnumerator()
		{
			return Items.GetEnumerator();
		}

		protected override object AddNewCore()
		{
			// Here we have control about instantiation of our new view model
			var newItem = new ProductItemViewModel();
			Add(newItem);

			return newItem;
		}

		public void CopyTo(IProductItemViewModel[] array, int arrayIndex)
		{
			for (int i = arrayIndex; i < array.Length; i++)
				array[i] = Items[i];
		}
	}
}
